## $Id: browser.py 11782 2014-08-29 05:31:07Z henrik $
##
## Copyright (C) 2011 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
"""UI components for basic applicants and related components.
"""
import grok
from zope.component import getUtility, createObject
from zope.security import checkPermission
from waeup.kofa.interfaces import IExtFileStore
from waeup.kofa.applicants.browser import (
    ApplicantRegistrationPage, ApplicantsContainerPage,
    ApplicationFeePaymentAddPage,
    OnlinePaymentApprovePage)
from waeup.kofa.applicants.interfaces import (
    ISpecialApplicant, IApplicantsUtils)
from kofacustom.nigeria.applicants.browser import (
    NigeriaApplicantDisplayFormPage,
    NigeriaApplicantManageFormPage,
    NigeriaPDFApplicationSlip)
from waeup.uniben.applicants.interfaces import (
    ICustomApplicant)

from waeup.uniben.interfaces import MessageFactory as _

PASTQ_ALL = ['ADT','EPCS','ESM','HEK','VTE']

PASTQ_AL = ['ENL','FAA','FOL','HIS','LAL',
            'PHL','THR','BUL','JIL','LAW','PPL','PUL'] + PASTQ_ALL

PASTQ_BS = ['ANT','ANY','CHH','COH','HAE','MED','MEH','PHS','SUR',
            'PCG','PCH','PCO', 'PCT','PHA','PHM','PMB','ANA','MBC',
            'MLS','NSC','PSY','DPV','ODR','OSP','PER', 'RES','AEB',
            'BCH','BOT','CED','EVL','MCB','OPT','PBB','SLT','ZOO',
            'AEE','ANS', 'CRS','FIS','FOW','SOS'] + PASTQ_ALL

PASTQ_EPS = ['CHE','CVE','DMIC','EEE','MCH','PEE','PRE','CHM',
             'CSC','GLY','MTH','PHY'] + PASTQ_ALL

PASTQ_MSS = ['ACC','BNK','BUS','ECO','GEO','POL','SAA','SWK'] + PASTQ_ALL

class CustomApplicantsContainerPage(ApplicantsContainerPage):
    """The standard view for regular applicant containers.
    """

    @property
    def form_fields(self):
        form_fields = super(CustomApplicantsContainerPage, self).form_fields
        usertype = getattr(self.request.principal, 'user_type', None)
        if self.request.principal.id == 'zope.anybody' or  \
            usertype in ('applicant', 'student'):
            return form_fields.omit('application_fee')
        return form_fields

class CustomApplicantRegistrationPage(ApplicantRegistrationPage):
    """Captcha'd registration page for applicants.
    """

    def _redirect(self, email, password, applicant_id):
        # Forward email and credentials to landing page.
        self.redirect(self.url(self.context, 'registration_complete',
            data = dict(email=email, password=password,
            applicant_id=applicant_id)))
        return

class CustomApplicantDisplayFormPage(NigeriaApplicantDisplayFormPage):
    """A display view for applicant data.
    """
    grok.template('applicantdisplaypage')

    def _show_pastq_putme(self):
        return self.target.startswith('putme') \
               and self.context.state in ('paid', 'submitted') \
               and getattr(self.context, 'course1') is not None

    @property
    def show_pastq_al(self):
        return self._show_pastq_putme() \
               and self.context.course1.__parent__.__parent__.code in PASTQ_AL

    @property
    def show_pastq_bs(self):
        return self._show_pastq_putme() \
               and self.context.course1.__parent__.__parent__.code in PASTQ_BS

    @property
    def show_pastq_eps(self):
        return self._show_pastq_putme() \
               and self.context.course1.__parent__.__parent__.code in PASTQ_EPS

    @property
    def show_pastq_mss(self):
        return self._show_pastq_putme() \
               and self.context.course1.__parent__.__parent__.code in PASTQ_MSS

    @property
    def show_pastq_pude(self):
        return self.target.startswith('pude') \
               and self.context.state in ('paid', 'submitted') 

    # customizations for admission checking payments
    @property
    def form_fields(self):
        if self.context.special:
            form_fields = grok.AutoFields(ISpecialApplicant).omit('locked')
        else:
            form_fields = grok.AutoFields(ICustomApplicant).omit(
                'locked', 'course_admitted', 'password', 'suspended')
        return form_fields

class CustomNigeriaPDFApplicationSlip(NigeriaPDFApplicationSlip):


    @property
    def note(self):
        if self.target is not None and not self.target.startswith('pg') \
            and not self._reduced_slip():
            return _(u'<br /><br /><br />'
                      'Comfirm your screening venue 72 hours to the screening. '
                      '<br /><br />'
                      'No bags, phones and calculators are allowed in the '
                      'screening venues.')
        return

class CustomApplicationFeePaymentAddPage(ApplicationFeePaymentAddPage):
    """ Page to add an online payment ticket
    """

    @property
    def custom_requirements(self):
        store = getUtility(IExtFileStore)
        if not store.getFileByContext(self.context, attr=u'passport.jpg'):
            return _('Upload your 1"x1" Red background passport photo before making payment.')
        return ''

    # customizations for admission checking payments
    def update(self):
        # Additional requirements in custom packages.
        if self.custom_requirements:
            self.flash(
                self.custom_requirements,
                type='danger')
            self.redirect(self.url(self.context))
            return
        if not self.context.special:
            for key in self.context.keys():
                ticket = self.context[key]
                if ticket.p_state == 'paid':
                      self.flash(
                          _('This type of payment has already been made.'),
                          type='warning')
                      self.redirect(self.url(self.context))
                      return
        applicants_utils = getUtility(IApplicantsUtils)
        container = self.context.__parent__
        payment = createObject(self.factory)
        failure = applicants_utils.setPaymentDetails(
            container, payment, self.context)
        if failure is not None:
            self.flash(failure[0], type='danger')
            self.redirect(self.url(self.context))
            return
        self.context[payment.p_id] = payment
        self.flash(_('Payment ticket created.'))
        self.redirect(self.url(payment))
        return


class CustomApplicantManageFormPage(NigeriaApplicantManageFormPage):

    @property
    def custom_upload_requirements(self):
        if not checkPermission('waeup.uploadPassportPictures', self.context):
            return _('You are not entitled to upload passport pictures.')

class CustomOnlinePaymentApprovePage(OnlinePaymentApprovePage):
    """ Approval view
    """

    # customizations for admission checking payments
    def update(self):
        flashtype, msg, log = self.context.approveApplicantPayment()
        if log is not None:
            applicant = self.context.__parent__
            # Add log message to applicants.log
            applicant.writeLogMessage(self, log)
            # Add log message to payments.log
            self.context.logger.info(
                '%s,%s,%s,%s,%s,,,,,,' % (
                applicant.applicant_id,
                self.context.p_id, self.context.p_category,
                self.context.amount_auth, self.context.r_code))
        self.flash(msg, type=flashtype)
        return
